@@ -16,7 +16,9 @@ PKGConfig.have_package('gdk-pixbuf-2.0') or exit 1
 
 setup_win32(PACKAGE_NAME)
 
-have_func("gdk_pixbuf_set_option")
+have_func("gdk_pixbuf_set_option", "gdk-pixbuf/gdk-pixbuf.h") do |src|
+  "#define GDK_PIXBUF_ENABLE_BACKEND\n#{src}"
+end
 have_header("gdk-pixbuf/gdk-pixbuf-io.h")
 
 if PKGConfig.have_package('cairo') and
@@ -27,7 +29,7 @@ if PKGConfig.have_package('cairo') and
       puts "Error! Set CAIRO_PATH."
       exit 1
     end
-    add_depend_package("cairo", "packages/cairo/ext", ENV["CAIRO_PATH"])
+    add_depend_package("cairo", "src", ENV["CAIRO_PATH"])
     $defs << "-DRUBY_CAIRO_PLATFORM_WIN32"
   end
 end
