@@ -4,14 +4,22 @@
   rbgobj_type.c -
 
   $Author: sakai $
-  $Date: 2003/09/20 01:27:39 $
+  $Date: 2007/07/16 03:35:53 $
   created at: Sun Jun  9 20:31:47 JST 2002
-
+ 
+  Copyright (C) 2002-2006  Ruby-GNOME2 Project Team
   Copyright (C) 2002,2003  Masahiro Sakai
 
 **********************************************************************/
 
-#include "global.h"
+#include "rbgprivate.h"
+
+#include "rbgprivate.h"
+#include <rubysig.h>
+
+/**********************************************************************/
+/* Type Mapping */
+
 #ifdef RBGOBJ_USE_DLPTR
 #include "dl.h"
 #endif
@@ -20,7 +28,21 @@ static ID id_new;
 static ID id_superclass;
 static VALUE gtype_to_cinfo;
 static VALUE klass_to_cinfo;
-static void rbgobj_init_interface(VALUE interface);
+
+static GHashTable* dynamic_gtype_list;
+typedef struct {
+    const gchar* name;
+    VALUE module;
+    void (*mark)(gpointer);
+    void (*free)(gpointer);
+    int flags; /* RGObjClassFlag */
+} RGObjClassInfoDynamic;
+
+typedef struct {
+    VALUE parent;
+    GType gtype;
+    gboolean create_class;
+} RGObjClassByGtypeData;
 
 static void
 cinfo_mark(RGObjClassInfo* cinfo)
@@ -40,16 +62,27 @@ rbgobj_lookup_class(klass)
         return cinfo;
     }
 
-    if (TYPE(klass) == T_CLASS)
-        return rbgobj_lookup_class(rb_funcall(klass, id_superclass, 0));
+    if (TYPE(klass) == T_CLASS) {
+        VALUE super;
+        if (FL_TEST(klass, FL_SINGLETON)) {
+            super = rb_class_real(klass);
+        } else {
+            super = rb_funcall(klass, id_superclass, 0);
+        }
+        return rbgobj_lookup_class(super);
+    }
 
-    rb_raise(rb_eRuntimeError, "can't get gobject class infomation");    
+    rb_raise(rb_eRuntimeError, "can't get gobject class information");    
 }
 
 static VALUE
-get_superclass(gtype)
-    GType gtype;
+get_superclass(GType gtype)
 {
+    VALUE super_class;
+
+    if (rbgobj_convert_get_superclass(gtype, &super_class))
+        return super_class;
+
     switch (gtype) {
       case G_TYPE_PARAM:
       case G_TYPE_OBJECT:
@@ -66,113 +99,211 @@ get_superclass(gtype)
       case G_TYPE_FLAGS:
         return rb_cObject;
       default:
-        {
-            const RGObjClassInfo* cinfo_super =
-                rbgobj_lookup_class_by_gtype(g_type_parent(gtype));
-            return cinfo_super->klass;
+      {
+          const RGObjClassInfo* cinfo_super =
+              rbgobj_lookup_class_by_gtype(g_type_parent(gtype), Qnil);
+          return cinfo_super->klass;
+      }
+    }
+}
+
+static VALUE
+rbgobj_lookup_class_by_gtype_body(VALUE data)
+{
+    GType fundamental_type;
+    RGObjClassInfo* cinfo;
+    RGObjClassInfoDynamic* cinfod;
+    void* gclass = NULL;
+    VALUE c;
+    GType gtype;
+    VALUE parent;
+    RGObjClassByGtypeData *cdata = (RGObjClassByGtypeData *)data;
+
+    gtype = cdata->gtype;
+    parent = cdata->parent;
+
+    if (gtype == G_TYPE_INVALID)
+        return (VALUE)NULL;
+
+    c = rb_hash_aref(gtype_to_cinfo, INT2NUM(gtype));
+    if (!NIL_P(c)) {
+        Data_Get_Struct(c, RGObjClassInfo, cinfo);
+        return (VALUE)cinfo;
+    }
+
+    if (!cdata->create_class)
+	return (VALUE)NULL;
+
+    c = Data_Make_Struct(rb_cData, RGObjClassInfo, cinfo_mark, NULL, cinfo);
+    cinfo->gtype = gtype;
+    cinfo->mark  = NULL;
+    cinfo->free  = NULL;
+    cinfo->flags = 0;
+
+    fundamental_type = G_TYPE_FUNDAMENTAL(gtype);
+    switch (fundamental_type){
+    case G_TYPE_POINTER:
+    case G_TYPE_BOXED:
+    case G_TYPE_PARAM:
+    case G_TYPE_OBJECT:
+    case G_TYPE_ENUM:
+    case G_TYPE_FLAGS:
+        if (NIL_P(parent)) parent = get_superclass(gtype);
+        cinfo->klass = rb_funcall(rb_cClass, id_new, 1, parent);
+        break;
+        
+    case G_TYPE_INTERFACE:
+        cinfo->klass = rb_module_new();
+        break;
+        
+    default:
+      if (NIL_P(parent)) parent = get_superclass(gtype);
+      if (NIL_P(parent)) {
+          fprintf(stderr,
+                  "%s: %s's fundamental type %s isn't supported\n",
+                  "rbgobj_lookup_class_by_gtype",
+                  g_type_name(gtype),
+                  g_type_name(fundamental_type));
+          return (VALUE)NULL;
+      }
+      cinfo->klass = rb_funcall(rb_cClass, id_new, 1, parent);
+    }
+
+    cinfod = (RGObjClassInfoDynamic*)g_hash_table_lookup(dynamic_gtype_list, g_type_name(gtype));
+    if (cinfod){
+        cinfo->mark = cinfod->mark;
+        cinfo->free = cinfod->free;
+        rb_define_const(cinfod->module, cinfod->name, cinfo->klass);
+    }
+
+    rb_hash_aset(klass_to_cinfo, cinfo->klass, c);
+    rb_hash_aset(gtype_to_cinfo, INT2NUM(gtype), c);
+    
+    if (G_TYPE_IS_CLASSED(gtype))
+        gclass = g_type_class_ref(gtype);
+    
+    if (G_TYPE_IS_INSTANTIATABLE(gtype) || G_TYPE_IS_INTERFACE(gtype))
+        rbgobj_define_action_methods(cinfo->klass);
+    
+    if (G_TYPE_IS_INSTANTIATABLE(gtype)){
+        GType* interfaces = NULL;
+        guint n_interfaces = 0;
+        int i;
+        
+        interfaces = g_type_interfaces(gtype, &n_interfaces);
+        for (i = 0; i < n_interfaces; i++){
+            rb_include_module(
+                cinfo->klass,
+                rbgobj_lookup_class_by_gtype(interfaces[i], Qnil)->klass);
+        }
+        g_free(interfaces);
+    }
+
+    if (!rbgobj_convert_type_init_hook(gtype, cinfo->klass)) {
+        switch (fundamental_type) {
+          case G_TYPE_OBJECT:
+            rbgobj_init_object_class(cinfo->klass);
+            break;
+          case G_TYPE_ENUM:
+            rbgobj_init_enum_class(cinfo->klass);
+            break;
+          case G_TYPE_FLAGS:
+            rbgobj_init_flags_class(cinfo->klass);
+            break;
+          case G_TYPE_INTERFACE:
+            rbgobj_init_interface(cinfo->klass);
+            break;
+          default:
+            rbgobj_convert_type_init_hook(fundamental_type, cinfo->klass);
+            break;
         }
     }
+
+    if (gclass)
+        g_type_class_unref(gclass);
+
+    return (VALUE)cinfo;
+}
+
+static VALUE
+rbgobj_lookup_class_by_gtype_ensure(VALUE value)
+{
+    rb_thread_critical = (int)value;
+    return Qundef;
 }
 
 const RGObjClassInfo *
-rbgobj_lookup_class_by_gtype(gtype)
-    GType gtype;
+rbgobj_lookup_class_by_gtype(GType gtype, VALUE parent)
 {
-    RGObjClassInfo* cinfo;
-    VALUE c = rb_hash_aref(gtype_to_cinfo, INT2NUM(gtype));
+    return rbgobj_lookup_class_by_gtype_full(gtype, parent, TRUE);
+}
 
-    if (!NIL_P(c)){
-        Data_Get_Struct(c, RGObjClassInfo, cinfo);
-    } else {
-        void* gclass = NULL;
+const RGObjClassInfo *
+rbgobj_lookup_class_by_gtype_full(GType gtype, VALUE parent,
+				  gboolean create_class)
+{
+    VALUE critical = rb_thread_critical;
+    RGObjClassByGtypeData data;
 
-        c = Data_Make_Struct(rb_cData, RGObjClassInfo, cinfo_mark, NULL, cinfo);
+    data.gtype = gtype;
+    data.parent = parent;
+    data.create_class = create_class;
 
-        switch (G_TYPE_FUNDAMENTAL(gtype)){
-          case G_TYPE_POINTER:
-          case G_TYPE_BOXED:
-          case G_TYPE_PARAM:
-          case G_TYPE_OBJECT:
-          case G_TYPE_ENUM:
-          case G_TYPE_FLAGS:
-            cinfo->klass = rb_funcall(rb_cClass, id_new, 1,
-                                      get_superclass(gtype));
-            break;
+    rb_thread_critical = 1;
 
-          case G_TYPE_INTERFACE:
-            cinfo->klass = rb_module_new();
-            break;
-
-          default:
-            /* we should raise exception? */
-            fprintf(stderr,
-                    "%s: %s's fundamental type %s isn't supported\n",
-                    "rbgobj_lookup_class_by_gtype",
-                    g_type_name(gtype),
-                    g_type_name(G_TYPE_FUNDAMENTAL(gtype)));
-            return NULL;
-        }
-
-        cinfo->gtype = gtype;
-        cinfo->mark  = NULL;
-        cinfo->free  = NULL;
-        cinfo->flags = 0;
-
-        rb_hash_aset(klass_to_cinfo, cinfo->klass, c);
-        rb_hash_aset(gtype_to_cinfo, INT2NUM(gtype), c);
-
-        if (G_TYPE_IS_CLASSED(gtype))
-            gclass = g_type_class_ref(gtype);
-
-        if (G_TYPE_IS_INSTANTIATABLE(gtype) || G_TYPE_IS_INTERFACE(gtype))
-            rbgobj_define_action_methods(cinfo->klass);
-
-        if (G_TYPE_IS_INSTANTIATABLE(gtype)){
-            GType* interfaces = NULL;
-            guint n_interfaces = 0;
-            int i;
-
-            interfaces = g_type_interfaces(gtype, &n_interfaces);
-            for (i = 0; i < n_interfaces; i++){
-                rb_include_module(
-                    cinfo->klass,
-                    rbgobj_lookup_class_by_gtype(interfaces[i])->klass);
-            }
-            g_free(interfaces);
-        }
-
-        if (G_TYPE_FUNDAMENTAL(gtype) == G_TYPE_OBJECT) {
-            rbgobj_define_property_accessors(cinfo->klass);
-        } else if (G_TYPE_FUNDAMENTAL(gtype) == G_TYPE_ENUM) {
-            rbgobj_init_enum_class(cinfo->klass);
-        } else if (G_TYPE_FUNDAMENTAL(gtype) == G_TYPE_FLAGS) {
-            rbgobj_init_flags_class(cinfo->klass);
-        } else if (G_TYPE_FUNDAMENTAL(gtype) == G_TYPE_INTERFACE) {
-            rbgobj_init_interface(cinfo->klass);
-        }
-
-        if (gclass)
-            g_type_class_unref(gclass);
-    }
-
-    return cinfo;
+    return (RGObjClassInfo *)rb_ensure(rbgobj_lookup_class_by_gtype_body,
+				       (VALUE)&data,
+				       rbgobj_lookup_class_by_gtype_ensure,
+				       critical);
 }
 
 VALUE
-rbgobj_define_class(gtype, name, module, mark, free)
+rbgobj_gtype_to_ruby_class(GType gtype)
+{
+    const RGObjClassInfo *cinfo;
+
+    cinfo = GTYPE2CINFO(gtype);
+    return cinfo ? cinfo->klass : Qnil;
+}
+
+VALUE
+rbgobj_define_class(gtype, name, module, mark, free, parent)
     GType gtype;
     const gchar* name;
     VALUE module;
     void* mark;
     void* free;
+    VALUE parent;
 {
-    RGObjClassInfo* cinfo = (RGObjClassInfo*)rbgobj_lookup_class_by_gtype(gtype);
+    RGObjClassInfo* cinfo;
+    if (gtype == 0)
+        rb_bug("rbgobj_define_class: Invalid gtype [%s]\n", name);
+
+    cinfo = (RGObjClassInfo*)rbgobj_lookup_class_by_gtype(gtype, parent);
     cinfo->mark = mark;
     cinfo->free = free;
     rb_define_const(module, name, cinfo->klass);
     return cinfo->klass;
 }
 
+VALUE
+rbgobj_define_class_dynamic(gtype_name, name, module, mark, free)
+    const gchar* gtype_name;
+    const gchar* name;
+    VALUE module;
+    void* mark;
+    void* free;
+{
+    RGObjClassInfoDynamic* cinfo;
+    cinfo = (RGObjClassInfoDynamic*)g_new(RGObjClassInfoDynamic, 1);
+    cinfo->name = name;
+    cinfo->module = module;
+    cinfo->mark = mark;
+    cinfo->free = free;
+    g_hash_table_insert(dynamic_gtype_list, (void*)gtype_name, (void*)cinfo);
+    return Qnil;
+}
+
 void
 rbgobj_register_class(VALUE klass,
                       GType gtype,
@@ -194,7 +325,53 @@ rbgobj_register_class(VALUE klass,
         rb_hash_aset(gtype_to_cinfo, INT2NUM(gtype), c);
 }
 
+#define _register_fundamental_klass_to_gtype(klass, gtype) \
+    rbgobj_register_class(klass, gtype, TRUE, FALSE)
+
+#define _register_fundamental_gtype_to_klass(gtype,klass) \
+    rbgobj_register_class(klass, gtype, FALSE, TRUE)
+
+static void
+Init_typemap()
+{
+    id_new = rb_intern("new");
+    id_superclass = rb_intern("superclass");
+
+    rb_global_variable(&gtype_to_cinfo);
+    rb_global_variable(&klass_to_cinfo);
+    gtype_to_cinfo = rb_hash_new();
+    klass_to_cinfo = rb_hash_new();
+
+    _register_fundamental_klass_to_gtype(rb_cFixnum, G_TYPE_LONG);
+    _register_fundamental_klass_to_gtype(rb_cFloat, G_TYPE_DOUBLE);
+    _register_fundamental_klass_to_gtype(rb_cInteger, G_TYPE_LONG);
+    _register_fundamental_klass_to_gtype(rb_cString, G_TYPE_STRING);
+    _register_fundamental_klass_to_gtype(rb_cSymbol, G_TYPE_STRING);
+    _register_fundamental_klass_to_gtype(Qnil, G_TYPE_NONE);
+    _register_fundamental_klass_to_gtype(rb_cNilClass, G_TYPE_NONE);
+    _register_fundamental_klass_to_gtype(rb_cTrueClass, G_TYPE_BOOLEAN);
+    _register_fundamental_klass_to_gtype(rb_cFalseClass, G_TYPE_BOOLEAN);
+    _register_fundamental_klass_to_gtype(Qtrue, G_TYPE_BOOLEAN);
+    _register_fundamental_klass_to_gtype(Qfalse, G_TYPE_BOOLEAN);
+    _register_fundamental_klass_to_gtype(rb_cObject, RBGOBJ_TYPE_RUBY_VALUE);
+
+    _register_fundamental_gtype_to_klass(G_TYPE_UINT, rb_cInteger);
+    _register_fundamental_gtype_to_klass(G_TYPE_FLOAT, rb_cFloat);
+    _register_fundamental_gtype_to_klass(G_TYPE_DOUBLE, rb_cFloat);
+    _register_fundamental_gtype_to_klass(G_TYPE_INT64, rb_cInteger);
+    _register_fundamental_gtype_to_klass(G_TYPE_UINT64, rb_cInteger);
+    _register_fundamental_gtype_to_klass(G_TYPE_INT, rb_cInteger);
+    _register_fundamental_gtype_to_klass(G_TYPE_LONG, rb_cInteger);
+    _register_fundamental_gtype_to_klass(G_TYPE_CHAR, rb_cFixnum);
+    _register_fundamental_gtype_to_klass(G_TYPE_UCHAR, rb_cFixnum);
+    _register_fundamental_gtype_to_klass(G_TYPE_STRING, rb_cString);
+    _register_fundamental_gtype_to_klass(G_TYPE_ULONG, rb_cInteger);
+    _register_fundamental_gtype_to_klass(G_TYPE_NONE, rb_cNilClass);
+    _register_fundamental_gtype_to_klass(G_TYPE_BOOLEAN, rb_cTrueClass);
+}
+
 /**********************************************************************/
+/* GLib::Type */
 
 VALUE rbgobj_cType;
 static ID id_gtype;
@@ -213,9 +390,12 @@ GType
 rbgobj_gtype_get(self)
     VALUE self;
 {
-    if (!RTEST(rb_obj_is_kind_of(self, rbgobj_cType)))
-        rb_raise(rb_eTypeError, "Not a GLib::Type");
-    return NUM2ULONG(rb_ivar_get(self, id_gtype));
+    if (RVAL2CBOOL(rb_obj_is_kind_of(self, rbgobj_cType))) {
+        return NUM2ULONG(rb_ivar_get(self, id_gtype));
+    } else {
+        return CLASS2GTYPE(self);
+    }
+    rb_raise(rb_eTypeError, "Not a GLib::Type");
 }
 
 static VALUE
@@ -224,13 +404,12 @@ type_initialize(self, type)
 {
     GType gtype;
 
-    if (RTEST(rb_obj_is_kind_of(type, rb_cInteger))) {
+    if (RVAL2CBOOL(rb_obj_is_kind_of(type, rb_cInteger))) {
         gtype = NUM2UINT(type);
         // XXX
         if (!g_type_name(gtype))
             gtype = G_TYPE_INVALID;
     } else {
-        StringValue(type);
         gtype = g_type_from_name(StringValuePtr(type));
     }
 
@@ -261,7 +440,7 @@ static VALUE
 type_compare(self, other)
     VALUE self, other;
 {
-    if (!RTEST(rb_obj_is_kind_of(other, rbgobj_cType)))
+    if (!RVAL2CBOOL(rb_obj_is_kind_of(other, rbgobj_cType)))
         return Qnil;
     else {
         GType a = rbgobj_gtype_get(self);
@@ -282,12 +461,12 @@ static VALUE
 type_eq(self, other)
     VALUE self, other;
 {
-    if (!RTEST(rb_obj_is_kind_of(other, rbgobj_cType)))
+    if (!RVAL2CBOOL(rb_obj_is_kind_of(other, rbgobj_cType)))
         return Qnil;
     else {
         GType a = rbgobj_gtype_get(self);
         GType b = rbgobj_gtype_get(other);
-        return (a == b) ? Qtrue : Qfalse;
+        return CBOOL2RVAL(a == b);
     }
 }
 
@@ -295,12 +474,12 @@ static VALUE
 type_lt_eq(self, other)
     VALUE self, other;
 {
-    if (!RTEST(rb_obj_is_kind_of(other, rbgobj_cType)))
+    if (!RVAL2CBOOL(rb_obj_is_kind_of(other, rbgobj_cType)))
         return Qnil;
     else {
         GType a = rbgobj_gtype_get(self);
         GType b = rbgobj_gtype_get(other);
-        return g_type_is_a(a, b) ? Qtrue : Qfalse;
+        return CBOOL2RVAL(g_type_is_a(a, b));
     }
 }
 
@@ -308,12 +487,12 @@ static VALUE
 type_gt_eq(self, other)
     VALUE self, other;
 {
-    if (!RTEST(rb_obj_is_kind_of(other, rbgobj_cType)))
+    if (!RVAL2CBOOL(rb_obj_is_kind_of(other, rbgobj_cType)))
         return Qnil;
     else {
         GType a = rbgobj_gtype_get(self);
         GType b = rbgobj_gtype_get(other);
-        return g_type_is_a(b, a) ? Qtrue : Qfalse;
+        return CBOOL2RVAL(g_type_is_a(b, a));
     }
 }
 
@@ -321,12 +500,12 @@ static VALUE
 type_lt(self, other)
     VALUE self, other;
 {
-    if (!RTEST(rb_obj_is_kind_of(other, rbgobj_cType)))
+    if (!RVAL2CBOOL(rb_obj_is_kind_of(other, rbgobj_cType)))
         return Qnil;
     else {
         GType a = rbgobj_gtype_get(self);
         GType b = rbgobj_gtype_get(other);
-        return (g_type_is_a(a, b) && a != b) ? Qtrue : Qfalse;
+        return CBOOL2RVAL(g_type_is_a(a, b) && a != b);
     }
 }
 
@@ -334,12 +513,12 @@ static VALUE
 type_gt(self, other)
     VALUE self, other;
 {
-    if (!RTEST(rb_obj_is_kind_of(other, rbgobj_cType)))
+    if (!RVAL2CBOOL(rb_obj_is_kind_of(other, rbgobj_cType)))
         return Qnil;
     else {
         GType a = rbgobj_gtype_get(self);
         GType b = rbgobj_gtype_get(other);
-        return (g_type_is_a(b, a) && a != b) ? Qtrue : Qfalse;
+        return CBOOL2RVAL(g_type_is_a(b, a) && a != b);
     }
 }
 
@@ -368,77 +547,77 @@ static VALUE
 type_is_fundamental(self)
     VALUE self;
 {
-    return G_TYPE_IS_FUNDAMENTAL(rbgobj_gtype_get(self)) ? Qtrue : Qfalse;
+    return CBOOL2RVAL(G_TYPE_IS_FUNDAMENTAL(rbgobj_gtype_get(self)));
 }
 
 static VALUE
 type_is_derived(self)
     VALUE self;
 {
-    return G_TYPE_IS_DERIVED(rbgobj_gtype_get(self)) ? Qtrue : Qfalse;
+    return CBOOL2RVAL(G_TYPE_IS_DERIVED(rbgobj_gtype_get(self)));
 }
 
 static VALUE
 type_is_interface(self)
     VALUE self;
 {
-    return G_TYPE_IS_INTERFACE(rbgobj_gtype_get(self)) ? Qtrue : Qfalse;
+    return CBOOL2RVAL(G_TYPE_IS_INTERFACE(rbgobj_gtype_get(self)));
 }
 
 static VALUE
 type_is_classed(self)
     VALUE self;
 {
-    return G_TYPE_IS_CLASSED(rbgobj_gtype_get(self)) ? Qtrue : Qfalse;
+    return CBOOL2RVAL(G_TYPE_IS_CLASSED(rbgobj_gtype_get(self)));
 }
 
 static VALUE
 type_is_instantiatable(self)
     VALUE self;
 {
-    return G_TYPE_IS_INSTANTIATABLE(rbgobj_gtype_get(self)) ? Qtrue : Qfalse;
+    return CBOOL2RVAL(G_TYPE_IS_INSTANTIATABLE(rbgobj_gtype_get(self)));
 }
 
 static VALUE
 type_is_derivable(self)
     VALUE self;
 {
-    return G_TYPE_IS_DERIVABLE(rbgobj_gtype_get(self)) ? Qtrue : Qfalse;
+    return CBOOL2RVAL(G_TYPE_IS_DERIVABLE(rbgobj_gtype_get(self)));
 }
 
 static VALUE
 type_is_deep_derivable(self)
     VALUE self;
 {
-    return G_TYPE_IS_DEEP_DERIVABLE(rbgobj_gtype_get(self)) ? Qtrue : Qfalse;
+    return CBOOL2RVAL(G_TYPE_IS_DEEP_DERIVABLE(rbgobj_gtype_get(self)));
 }
 
 static VALUE
 type_is_abstract(self)
     VALUE self;
 {
-    return G_TYPE_IS_ABSTRACT(rbgobj_gtype_get(self)) ? Qtrue : Qfalse;
+    return CBOOL2RVAL(G_TYPE_IS_ABSTRACT(rbgobj_gtype_get(self)));
 }
 
 static VALUE
 type_is_value_abstract(self)
     VALUE self;
 {
-    return G_TYPE_IS_VALUE_ABSTRACT(rbgobj_gtype_get(self)) ? Qtrue : Qfalse;
+    return CBOOL2RVAL(G_TYPE_IS_VALUE_ABSTRACT(rbgobj_gtype_get(self)));
 }
 
 static VALUE
 type_is_value_type(self)
     VALUE self;
 {
-    return G_TYPE_IS_VALUE_TYPE(rbgobj_gtype_get(self)) ? Qtrue : Qfalse;
+    return CBOOL2RVAL(G_TYPE_IS_VALUE_TYPE(rbgobj_gtype_get(self)));
 }
 
 static VALUE
 type_has_value_table(self)
     VALUE self;
 {
-    return G_TYPE_HAS_VALUE_TABLE(rbgobj_gtype_get(self)) ? Qtrue : Qfalse;
+    return CBOOL2RVAL(G_TYPE_HAS_VALUE_TABLE(rbgobj_gtype_get(self)));
 }
 
 static VALUE
@@ -476,7 +655,7 @@ static VALUE
 type_is_a(self, is_a_type)
     VALUE self, is_a_type;
 {
-    return g_type_is_a(rbgobj_gtype_get(self), rbgobj_gtype_get(is_a_type)) ? Qtrue : Qfalse;
+    return CBOOL2RVAL(g_type_is_a(rbgobj_gtype_get(self), rbgobj_gtype_get(is_a_type)));
 }
 
 #if 0
@@ -555,6 +734,7 @@ void _def_fundamental_type(VALUE ary, GType gtype, const char* name)
 static void
 Init_type()
 {
+    dynamic_gtype_list = g_hash_table_new(g_str_hash, g_str_equal);
     id_gtype = rb_intern("__gobject_gtype__");
 
     rbgobj_cType = rb_define_class_under(mGLib, "Type", rb_cObject);
@@ -627,138 +807,9 @@ Init_type()
 
 /**********************************************************************/
 
-VALUE mMetaInterface;
-
-static void 
-Init_interface_commons()
-{
-    mMetaInterface = rb_define_module_under(mGLib, "MetaInterface");
-    rb_define_method(mMetaInterface, "gtype", generic_s_gtype, 0);
-}
-
-/**********************************************************************/
-
-VALUE cInstantiatable;
-
-static VALUE
-instantiatable_s_allocate(klass)
-     VALUE klass;
-{
-     rb_raise(rb_eTypeError, "abstract class");
-}
-
-static VALUE
-instantiatable_get_gtype(self)
-    VALUE self;
-{
-    return rbgobj_gtype_new(G_TYPE_FROM_INSTANCE(rbgobj_instance_from_ruby_object(self)));
-}
-
-static VALUE
-instantiatable_clone(self)
-    VALUE self;
-{
-    rb_raise(rb_eTypeError, "can't clone %s", rb_class2name(CLASS_OF(self)));
-}
-
-static void
-Init_instantiatable()
-{
-    cInstantiatable = rb_define_class_under(mGLib, "Instantiatable", rb_cObject);
-    rb_extend_object(cInstantiatable, mMetaInterface);
-
-#ifndef HAVE_RB_DEFINE_ALLOC_FUNC
-    rb_define_singleton_method(cInstantiatable, "allocate", instantiatable_s_allocate, 0);
-#else
-    rb_define_alloc_func(cInstantiatable, instantiatable_s_allocate);
-#endif
-#ifndef HAVE_OBJECT_ALLOCATE
-    rb_define_singleton_method(cInstantiatable, "new", &generic_s_new, -1);
-#endif
-
-    rb_define_method(cInstantiatable, "gtype", instantiatable_get_gtype, 0);
-    rb_define_method(cInstantiatable, "clone", instantiatable_clone, 0);
-}
-
-/**********************************************************************/
-
-static VALUE
-interface_s_append_features(self, klass)
-     VALUE self, klass;
-{
-    if (!rb_obj_is_kind_of(klass, cInstantiatable))
-        rb_raise(rb_eTypeError, "Not a subclass of GLib::Instantiatable");
-    return rb_call_super(1, &klass);
-}
-
-static void
-rbgobj_init_interface(interface)
-    VALUE interface;
-{
-    rb_extend_object(interface, mMetaInterface);
-    rb_define_singleton_method(interface, "append_features",
-                               interface_s_append_features, 1);
-    /* pseudo inheritance */
-    if (CLASS2GTYPE(interface) != G_TYPE_INTERFACE)
-        rb_include_module(interface, GTYPE2CLASS(G_TYPE_INTERFACE));
-}
-
-static void
-Init_interface()
-{
-    G_DEF_INTERFACE(G_TYPE_INTERFACE, "Interface", mGLib);
-}
-
-/**********************************************************************/
-
-#define _register_fundamental_klass_to_gtype(klass, gtype) \
-    rbgobj_register_class(klass, gtype, TRUE, FALSE)
-
-#define _register_fundamental_gtype_to_klass(gtype,klass) \
-    rbgobj_register_class(klass, gtype, FALSE, TRUE)
-
-/*
- * Init
- */
 void Init_gobject_gtype()
 {
     g_type_init();
-
-    id_new = rb_intern("new");
-    id_superclass = rb_intern("superclass");
-
-    rb_global_variable(&gtype_to_cinfo);
-    rb_global_variable(&klass_to_cinfo);
-    gtype_to_cinfo = rb_hash_new();
-    klass_to_cinfo = rb_hash_new();
-
-    _register_fundamental_klass_to_gtype(rb_cFixnum, G_TYPE_LONG);
-    _register_fundamental_klass_to_gtype(rb_cFloat, G_TYPE_DOUBLE);
-    _register_fundamental_klass_to_gtype(rb_cInteger, G_TYPE_LONG);
-    _register_fundamental_klass_to_gtype(rb_cString, G_TYPE_STRING);
-    _register_fundamental_klass_to_gtype(rb_cSymbol, G_TYPE_STRING);
-    _register_fundamental_klass_to_gtype(rb_cNilClass, G_TYPE_NONE);
-    _register_fundamental_klass_to_gtype(rb_cTrueClass, G_TYPE_BOOLEAN);
-    _register_fundamental_klass_to_gtype(rb_cFalseClass, G_TYPE_BOOLEAN);
-    _register_fundamental_klass_to_gtype(rb_cObject, RBGOBJ_TYPE_RUBY_VALUE);
-
-    _register_fundamental_gtype_to_klass(G_TYPE_UINT, rb_cInteger);
-    _register_fundamental_gtype_to_klass(G_TYPE_FLOAT, rb_cFloat);
-    _register_fundamental_gtype_to_klass(G_TYPE_DOUBLE, rb_cFloat);
-    _register_fundamental_gtype_to_klass(G_TYPE_INT64, rb_cInteger);
-    _register_fundamental_gtype_to_klass(G_TYPE_UINT64, rb_cInteger);
-    _register_fundamental_gtype_to_klass(G_TYPE_INT, rb_cInteger);
-    _register_fundamental_gtype_to_klass(G_TYPE_LONG, rb_cInteger);
-    _register_fundamental_gtype_to_klass(G_TYPE_CHAR, rb_cFixnum);
-    _register_fundamental_gtype_to_klass(G_TYPE_UCHAR, rb_cFixnum);
-    _register_fundamental_gtype_to_klass(G_TYPE_STRING, rb_cString);
-    _register_fundamental_gtype_to_klass(G_TYPE_ULONG, rb_cInteger);
-    _register_fundamental_gtype_to_klass(G_TYPE_NONE, rb_cNilClass);
-    _register_fundamental_gtype_to_klass(G_TYPE_BOOLEAN, rb_cTrueClass);
-
+    Init_typemap();
     Init_type();
-
-    Init_interface_commons();
-    Init_instantiatable();
-    Init_interface();
 }
