@@ -3,8 +3,8 @@
 
   rbgobject.h -
 
-  $Author: ktou $
-  $Date: 2006/07/09 13:11:59 $
+  $Author: sakai $
+  $Date: 2007/07/19 22:03:44 $
 
   Copyright (C) 2003,2006  Ruby-GNOME2 Project Team
   Copyright (C) 2002,2003  Masahiro Sakai
@@ -30,8 +30,10 @@ extern "C" {
 
 #define G_DEF_CLASS(gtype, name, module)\
     (rbgobj_define_class(gtype, name, module, 0, 0, Qnil))
-#define G_DEF_CLASS2(gtype, name, module, mark, free)\
+#define G_DEF_CLASS_WITH_GC_FUNC(gtype, name, module, mark, free)	\
     (rbgobj_define_class(gtype, name, module, mark, free, Qnil))
+#define G_DEF_CLASS2(gtype, name, module, mark, free)			\
+    G_DEF_CLASS_WITH_GC_FUNC(gtype, name, module, mark, free)
 #define G_DEF_CLASS3(gtype_name, name, module)\
     (rbgobj_define_class_dynamic(gtype_name, name, module, 0, 0))
 #define G_DEF_CLASS4(gtype, name, module, parent) \
@@ -42,7 +44,7 @@ extern "C" {
 #define G_DEF_INTERFACE2(gtype, name, module, mark, free)\
     (rbgobj_define_class(gtype, name, module, mark, free, Qnil))
 
-#define G_DEF_FUNDAMENTAL(f) (rbgobj_fund_define_fundamental(f))
+#define RG_DEF_CONVERSION(table) (rbgobj_convert_define(table))
 
 #define G_RELATIVE(obj, rel) (rbgobj_add_relative(obj, rel))
 
@@ -74,14 +76,16 @@ RUBY_GLIB2_VAR  ID rbgobj_id_children;
 
 #define CLASS2CINFO(klass) (rbgobj_lookup_class(klass))
 #define GTYPE2CINFO(gtype) (rbgobj_lookup_class_by_gtype(gtype, Qnil))
+#define GTYPE2CINFO_NO_CREATE(gtype) (rbgobj_lookup_class_by_gtype_full(gtype, Qnil, FALSE))
 #define RVAL2CINFO(obj)    (rbgobj_lookup_class(CLASS_OF(obj)))
-#define GTYPE2CLASS(gtype) (GTYPE2CINFO(gtype)->klass)
+#define GTYPE2CLASS(gtype) (rbgobj_gtype_to_ruby_class(gtype))
 #define CLASS2GTYPE(klass) (rbgobj_lookup_class(klass)->gtype)
 #define RVAL2GTYPE(obj)    (CLASS2GTYPE(CLASS_OF(obj)))
 
 #define RVAL2GOBJ(obj)  (rbgobj_instance_from_ruby_object(obj))
 #define GOBJ2RVAL(gobj) (rbgobj_ruby_object_from_instance(gobj))
-#define GOBJ2RVALU(gobj) (rbgobj_ruby_object_from_instance_with_unref(gobj))
+#define GOBJ2RVAL_UNREF(gobj) (rbgobj_ruby_object_from_instance_with_unref(gobj))
+#define GOBJ2RVALU(gobj) GOBJ2RVAL_UNREF(gobj)
 #define GVAL2RVAL(v)    (rbgobj_gvalue_to_rvalue(v))
 
 #define RVAL2BOXED(obj, gtype)  (rbgobj_boxed_get(obj, gtype))
@@ -92,6 +96,9 @@ RUBY_GLIB2_VAR  ID rbgobj_id_children;
 #define GENUM2RVAL(n, gtype)    (rbgobj_make_enum(n, gtype))
 #define GFLAGS2RVAL(n, gtype)   (rbgobj_make_flags(n, gtype))
 
+#define RVAL2GPTR(object)       (rbgobj_ptr2cptr(object))
+#define GPTR2RVAL(ptr, gtype)   (rbgobj_ptr_new(gtype, ptr))
+
 #define G_DEF_CONSTANTS(mod, type, strip_prefix) \
 	rbgobj_add_constants(mod, type, strip_prefix)
 #define G_RENAME_CONSTANT(orig, alt) \
@@ -101,7 +108,7 @@ RUBY_GLIB2_VAR  ID rbgobj_id_children;
 
 typedef enum
 {
-    RBGOBJ_ABSTRACT_BUT_CREATABLE = 1 << 0,
+    RBGOBJ_ABSTRACT_BUT_CREATABLE = 1 << 0, /* deprecated */
     RBGOBJ_BOXED_NOT_COPY         = 1 << 1,
     RBGOBJ_DEFINED_BY_RUBY        = 1 << 2,
 } RGObjClassFlag;
@@ -123,6 +130,7 @@ extern VALUE rbgobj_ruby_object_from_instance2(gpointer instance, gboolean alloc
 extern VALUE rbgobj_ruby_object_from_instance_with_unref(gpointer instance);
 
 extern void rbgobj_add_relative(VALUE obj, VALUE relative);
+extern void rbgobj_invalidate_relatives(VALUE obj);
 extern void rbgobj_add_relative_removable(VALUE obj, VALUE relative,
                                           ID obj_ivar_id, VALUE hash_key);
 extern VALUE rbgobj_get_relative_removable(VALUE obj, ID obj_ivar_id,
@@ -131,8 +139,11 @@ extern void rbgobj_remove_relative(VALUE obj, ID obj_ivar_id, VALUE hash_key);
 extern void rbgobj_remove_relative_all(VALUE obj, ID obj_ivar_id);
 
 extern GObject* rbgobj_gobject_new(GType type, VALUE params_hash);
-extern VALUE rbgobj_create_object(VALUE klass);
+extern VALUE rbgobj_create_object(VALUE klass); /* deprecated */
 
+extern VALUE rbgobj_get_ruby_object_from_gobject(GObject* gobj, gboolean alloc);
+
+/* deprecated */
 extern void rbgobj_add_abstract_but_create_instance_class(GType gtype);
 
 /* rbgobj_typeinstance.c */
@@ -140,8 +151,12 @@ extern void rbgobj_gc_mark_instance(gpointer instance);
 
 
 /* rbgobj_type.c */
-extern const RGObjClassInfo* rbgobj_lookup_class(VALUE klass);
-extern const RGObjClassInfo* rbgobj_lookup_class_by_gtype(GType gtype, VALUE parent);
+extern const RGObjClassInfo *rbgobj_lookup_class(VALUE klass);
+extern const RGObjClassInfo *rbgobj_lookup_class_by_gtype(GType gtype, VALUE parent);
+extern const RGObjClassInfo *rbgobj_lookup_class_by_gtype_full(GType gtype,
+							       VALUE parent,
+							       gboolean create_object);
+extern VALUE rbgobj_gtype_to_ruby_class(GType gtype);
 extern VALUE rbgobj_define_class(GType gtype, const gchar* name, VALUE module,
                                  void* mark, void* free, VALUE parent); 
 extern VALUE rbgobj_define_class_dynamic(const gchar* gtype_name, 
@@ -170,6 +185,7 @@ extern void g_rclosure_attach(GClosure *closure, VALUE object);
 /* rbgobj_value.c */
 extern VALUE rbgobj_gvalue_to_rvalue(const GValue* value);
 extern void rbgobj_rvalue_to_gvalue(VALUE val, GValue* result);
+extern void rbgobj_initialize_gvalue(GValue *result, VALUE value);
 
 typedef void (*RValueToGValueFunc)(VALUE from, GValue* to);
 typedef VALUE (*GValueToRValueFunc)(const GValue* from);
@@ -192,12 +208,10 @@ extern void rbgobj_register_property_setter(GType gtype, const char* prop_name,
 extern void rbgobj_register_property_getter(GType gtype, const char* prop_name, GValueToRValueFunc func);
 
 /* rbgobj_boxed.c */
-extern VALUE rbgobj_boxed_create(VALUE klass);
+extern VALUE rbgobj_boxed_create(VALUE klass); /* deprecated */
 extern gpointer rbgobj_boxed_get(VALUE obj, GType gtype);
 extern VALUE rbgobj_make_boxed(gpointer data, GType gtype);
 extern void rbgobj_boxed_not_copy_obj(GType gtype);
-extern void rbgobj_boxed_ref(VALUE obj, VALUE other);
-extern void rbgobj_boxed_unref(VALUE obj, VALUE other);
 
 /* rbgobj_enums.c */
 extern void rbgobj_constant_remap(const char *original, const char *replacement);
@@ -212,8 +226,6 @@ extern void rbgobj_define_const(VALUE mod, const char *name, VALUE value);
 /* rbglib_mainloop.c */
 #define G_TYPE_MAIN_LOOP (g_main_loop_get_type())
 extern GType g_main_loop_get_type(void);
-extern void rbg_remove_internal_poll_func(VALUE data);
-extern void rbg_set_internal_poll_func(void);
 
 /* rbglib_maincontext.c */
 #define G_TYPE_MAIN_CONTEXT (g_main_context_get_type())
@@ -223,30 +235,26 @@ extern GType g_main_context_get_type(void);
 extern GType g_source_get_type(void);
 extern GType g_poll_fd_get_type(void);
 
-/* rbgobj_fundamental.c */
+/* rbgobj_convert.c */
 typedef struct {
-  GType type;
-  VALUE (*get_superclass)(void); 
-  void (*type_init_hook)(VALUE);
-  void (*rvalue2gvalue)(VALUE val, GValue *result);
-  VALUE (*gvalue2rvalue)(const GValue *);
-  void (*initialize)(VALUE, gpointer);
-  gpointer (*robj2instance)(VALUE);
-  VALUE (*instance2robj)(gpointer);
-} RGFundamental;
+    GType type;
+    VALUE klass;
+    VALUE (*get_superclass)(void);
+    void (*type_init_hook)(VALUE);
+    void (*rvalue2gvalue)(VALUE val, GValue *result);
+    VALUE (*gvalue2rvalue)(const GValue *);
+    void (*initialize)(VALUE, gpointer);
+    gpointer (*robj2instance)(VALUE);
+    VALUE (*instance2robj)(gpointer);
+    void (*unref)(gpointer instance);
+} RGConvertTable;
 
-extern void rbgobj_fund_define_fundamental(RGFundamental *f);
-extern gboolean rbgobj_fund_has_type(GType type);
-extern VALUE rbgobj_fund_get_superclass(GType type);
-extern void rbgobj_fund_type_init_hook(GType type, VALUE klass);
-extern VALUE rbgobj_fund_gvalue2rvalue(GType type, const GValue *value);
-extern gboolean rbgobj_fund_rvalue2gvalue(GType type, VALUE val, GValue *result);
-extern void rbgobj_fund_initialize(GType type, VALUE obj, gpointer cobj);
-extern gpointer rbgobj_fund_robj2instance(GType type, VALUE obj);
-extern VALUE rbgobj_fund_instance2robj(GType type, gpointer instance);
+extern void rbgobj_convert_define(RGConvertTable *table);
 
 #ifdef __cplusplus
 }
 #endif /* __cplusplus */
 
 #endif /* __RBGOBJECT_H__ */
+
+#include "rbgcompat.h"
