@@ -3,10 +3,10 @@
 
   rbgtkwidget.c -
 
-  $Author: mutoh $
-  $Date: 2004/03/05 16:24:30 $
+  $Author: ggc $
+  $Date: 2007/07/13 16:07:32 $
 
-  Copyright (C) 2002-2004 Ruby-GNOME2 Project Team
+  Copyright (C) 2002-2006 Ruby-GNOME2 Project Team
   Copyright (C) 1998-2000 Yukihiro Matsumoto,
                           Daisuke Kanda,
                           Hiroshi Igarashi
@@ -28,6 +28,7 @@ widget_get_flags(self)
 static VALUE
 widget_set_flags(self, flag)
     VALUE self;
+    VALUE flag;
 {
     GTK_WIDGET_SET_FLAGS(_SELF(self), RVAL2GFLAGS(flag, GTK_TYPE_WIDGET_FLAGS));
     return self;
@@ -36,6 +37,7 @@ widget_set_flags(self, flag)
 static VALUE
 widget_unset_flags(self, flag)
     VALUE self;
+    VALUE flag;
 {
     GTK_WIDGET_UNSET_FLAGS(_SELF(self), RVAL2GFLAGS(flag, GTK_TYPE_WIDGET_FLAGS));
     return self;
@@ -138,6 +140,16 @@ widget_queue_resize(self)
     return self;
 }
 
+#if GTK_CHECK_VERSION(2,4,0)
+static VALUE
+widget_queue_resize_no_redraw(self)
+    VALUE self;
+{
+    gtk_widget_queue_resize_no_redraw(_SELF(self));
+    return self;
+}
+#endif
+
 /* Note this method is not
    gtk_widget_get_size_request */
 static VALUE
@@ -184,10 +196,10 @@ static VALUE
 widget_remove_accelerator(self, accel, key, mod)
     VALUE self, accel, key, mod;
 {
-    return gtk_widget_remove_accelerator(_SELF(self),
-                                         GTK_ACCEL_GROUP(RVAL2GOBJ(accel)),
-                                         NUM2INT(key),
-                                         RVAL2GFLAGS(mod, GDK_TYPE_MODIFIER_TYPE)) ? Qtrue : Qfalse;
+    return CBOOL2RVAL(gtk_widget_remove_accelerator(_SELF(self),
+                                                    GTK_ACCEL_GROUP(RVAL2GOBJ(accel)),
+                                                    NUM2INT(key),
+                                                    RVAL2GFLAGS(mod, GDK_TYPE_MODIFIER_TYPE)));
 }
 
 static VALUE
@@ -203,21 +215,30 @@ static VALUE
 widget_list_accel_closures(self)
     VALUE self;
 {
-    return GLIST2ARY2(gtk_widget_list_accel_closures(_SELF(self)), G_TYPE_CLOSURE);
+    return GLIST2ARY2F(gtk_widget_list_accel_closures(_SELF(self)), G_TYPE_CLOSURE);
 }
 
+#if GTK_CHECK_VERSION(2,4,0)
+static VALUE
+widget_can_activate_accel(self, signal_id)
+    VALUE self, signal_id;
+{
+    return CBOOL2RVAL(gtk_widget_can_activate_accel(_SELF(self), NUM2UINT(signal_id)));
+}
+#endif
+
 static VALUE
 widget_event(self, event)
     VALUE self, event;
 {
-    return gtk_widget_event(_SELF(self), RVAL2GEV(event)) ? Qtrue : Qfalse;
+    return CBOOL2RVAL(gtk_widget_event(_SELF(self), RVAL2GEV(event)));
 }
 
 static VALUE
 widget_activate(self)
     VALUE self;
 {
-    return gtk_widget_activate(_SELF(self)) ? Qtrue : Qfalse;
+    return CBOOL2RVAL(gtk_widget_activate(_SELF(self)));
 }
 
 static VALUE
@@ -243,7 +264,7 @@ static VALUE
 widget_is_focus(self)
     VALUE self;
 {
-    return gtk_widget_is_focus(_SELF(self)) ? Qtrue : Qfalse;
+    return CBOOL2RVAL(gtk_widget_is_focus(_SELF(self)));
 }
 
 static VALUE
@@ -319,7 +340,7 @@ static VALUE
 widget_set_colormap(self, colormap)
     VALUE self, colormap;
 {
-    gtk_widget_set_colormap(_SELF(self), GDK_COLORMAP(RVAL2GOBJ(self)));
+    gtk_widget_set_colormap(_SELF(self), GDK_COLORMAP(RVAL2GOBJ(colormap)));
     return self;
 }
 
@@ -344,7 +365,7 @@ static VALUE
 widget_is_ancestor(self, ancestor)
     VALUE self, ancestor;
 {
-    return (gtk_widget_is_ancestor(_SELF(self), _SELF(ancestor)) ? Qtrue : Qfalse);
+    return CBOOL2RVAL(gtk_widget_is_ancestor(_SELF(self), _SELF(ancestor)));
 }
 
 static VALUE
@@ -367,7 +388,7 @@ static VALUE
 widget_hide_on_delete(self)
 	VALUE self;
 {
-    return gtk_widget_hide_on_delete(_SELF(self)) ? Qtrue: Qfalse;
+    return CBOOL2RVAL(gtk_widget_hide_on_delete(_SELF(self)));
 }
 
 static VALUE
@@ -472,6 +493,19 @@ widget_shape_combine_mask(self, shape_mask, offset_x, offset_y)
     return self;
 }
 
+#if GTK_CHECK_VERSION(2,10,0)
+static VALUE
+widget_input_shape_combine_mask(self, shape_mask, offset_x, offset_y)
+    VALUE self, shape_mask, offset_x, offset_y;
+{
+    gtk_widget_input_shape_combine_mask(_SELF(self),
+                                        GDK_BITMAP(RVAL2GOBJ(shape_mask)),
+                                        NUM2INT(offset_x),
+                                        NUM2INT(offset_y));
+    return self;
+}
+#endif
+
 static VALUE
 widget_path(self)
     VALUE self;
@@ -536,7 +570,7 @@ widget_modify_fg(self, state, color)
     VALUE self, state, color;
 {
     gtk_widget_modify_fg(_SELF(self), RVAL2GENUM(state, GTK_TYPE_STATE_TYPE), 
-                         (GdkColor*)RVAL2BOXED(color, GDK_TYPE_COLOR));
+                         RVAL2GDKCOLOR(color));
     return self;
 }
 
@@ -545,7 +579,7 @@ widget_modify_bg(self, state, color)
     VALUE self, state, color;
 {
     gtk_widget_modify_bg(_SELF(self), RVAL2GENUM(state, GTK_TYPE_STATE_TYPE),
-                         (GdkColor*)RVAL2BOXED(color, GDK_TYPE_COLOR));
+                         RVAL2GDKCOLOR(color));
     return self;
 }
 
@@ -554,7 +588,7 @@ widget_modify_text(self, state, color)
     VALUE self, state, color;
 {
     gtk_widget_modify_text(_SELF(self), RVAL2GENUM(state, GTK_TYPE_STATE_TYPE),
-                         (GdkColor*)RVAL2BOXED(color, GDK_TYPE_COLOR));
+                         RVAL2GDKCOLOR(color));
     return self;
 }
 
@@ -563,7 +597,7 @@ widget_modify_base(self, state, color)
     VALUE self, state, color;
 {
     gtk_widget_modify_base(_SELF(self), RVAL2GENUM(state, GTK_TYPE_STATE_TYPE),
-                         (GdkColor*)RVAL2BOXED(color, GDK_TYPE_COLOR));
+                         RVAL2GDKCOLOR(color));
     return self;
 }
 
@@ -576,11 +610,23 @@ widget_modify_font(self, font_desc)
     return self;
 }
 
+#if GTK_CHECK_VERSION(2,12,0)
+static VALUE
+widget_modify_cursor(self, primary, seconday)
+    VALUE self, primary, seconday;
+{
+    gtk_widget_modify_cursor(_SELF(self), 
+                             RVAL2BOXED(primary, GDK_TYPE_COLOR),
+                             RVAL2BOXED(seconday, GDK_TYPE_COLOR));
+    return self;
+}
+#endif
+
 static VALUE
 widget_create_pango_context(self)
     VALUE self;
 {
-    return GOBJ2RVAL(gtk_widget_create_pango_context(_SELF(self)));
+    return GOBJ2RVALU(gtk_widget_create_pango_context(_SELF(self)));
 }
 
 static VALUE
@@ -598,16 +644,20 @@ widget_create_pango_layout(argc, argv, self)
 {
     VALUE text;
     rb_scan_args(argc, argv, "01", &text);
-    return GOBJ2RVAL(gtk_widget_create_pango_layout(_SELF(self), 
-                                                    NIL_P(text) ? NULL : RVAL2CSTR(text)));
+    return GOBJ2RVALU(gtk_widget_create_pango_layout(_SELF(self), RVAL2CSTR2(text)));
 }
 
 static VALUE
-widget_render_icon(self, stock_id, size, detail)
-    VALUE self, stock_id, size, detail;
+widget_render_icon(argc, argv, self)
+    int argc;
+    VALUE* argv;
+    VALUE self;
 {
-    return GOBJ2RVAL(gtk_widget_render_icon(_SELF(self),  rb_id2name(SYM2ID(stock_id)),
-                                            RVAL2GENUM(size, GTK_TYPE_ICON_SIZE), RVAL2CSTR(detail)));
+    VALUE stock_id, size, detail;
+    rb_scan_args(argc, argv, "21", &stock_id, &size, &detail);
+    return GOBJ2RVALU(gtk_widget_render_icon(_SELF(self), rb_id2name(SYM2ID(stock_id)),
+                                             RVAL2GENUM(size, GTK_TYPE_ICON_SIZE),
+                                             RVAL2CSTR2(detail)));
 }
 
 static VALUE
@@ -647,7 +697,7 @@ static VALUE
 widget_set_double_buffered(self, double_buffered)
     VALUE self, double_buffered;
 {
-    gtk_widget_set_double_buffered(_SELF(self), RTEST(double_buffered));
+    gtk_widget_set_double_buffered(_SELF(self), RVAL2CBOOL(double_buffered));
     return self;
 }
 
@@ -655,7 +705,7 @@ static VALUE
 widget_set_redraw_on_allocate(self, redraw_on_allocate)
     VALUE self, redraw_on_allocate;
 {
-    gtk_widget_set_redraw_on_allocate(_SELF(self), RTEST(redraw_on_allocate));
+    gtk_widget_set_redraw_on_allocate(_SELF(self), RVAL2CBOOL(redraw_on_allocate));
     return self;
 }
 
@@ -671,17 +721,16 @@ static VALUE
 widget_set_scroll_adjustments(self, h, v)
     VALUE self, h, v;
 {
-    gboolean res = gtk_widget_set_scroll_adjustments(_SELF(self),
-                                                     GTK_ADJUSTMENT(_SELF(h)),
-                                                     GTK_ADJUSTMENT(_SELF(v)));
-    return res ? Qtrue : Qfalse;
+    return CBOOL2RVAL(gtk_widget_set_scroll_adjustments(_SELF(self),
+                                                        GTK_ADJUSTMENT(_SELF(h)),
+                                                        GTK_ADJUSTMENT(_SELF(v))));
 }
 
 static VALUE
 widget_mnemonic_activate(self, group_cycling)
     VALUE self, group_cycling;
 {
-    return gtk_widget_mnemonic_activate(_SELF(self), RTEST(group_cycling)) ? Qtrue : Qfalse;
+    return CBOOL2RVAL(gtk_widget_mnemonic_activate(_SELF(self), RVAL2CBOOL(group_cycling)));
 }
 
 /*
@@ -702,7 +751,7 @@ rc_property_parser(pspec, rc_string, property_value)
     VALUE ret = rb_funcall(func, id_call, 2, spec, CSTR2RVAL(rc_string->str));
     if (NIL_P(ret)) {
         return FALSE;
-    } else if (RTEST(ret)){
+    } else if (RVAL2CBOOL(ret)){
         rbgobj_rvalue_to_gvalue(ret, property_value);
         return TRUE;
     } else {
@@ -724,9 +773,10 @@ widget_s_install_style_property(self, spec)
 
     gclass = (GtkWidgetClass *)g_type_class_ref(cinfo->gtype);
     if (rb_block_given_p()){
-        VALUE func = G_BLOCK_PROC();
+        VALUE func = rb_block_proc();
         rb_hash_aset(style_prop_func_table, spec, func);
-        gtk_widget_class_install_style_property_parser(gclass, pspec, rc_property_parser);
+        gtk_widget_class_install_style_property_parser(gclass, pspec, 
+                                                       (GtkRcPropertyParser)rc_property_parser);
     } else {
         gtk_widget_class_install_style_property(gclass, pspec); 
     }
@@ -770,7 +820,7 @@ widget_s_style_properties(argc, argv, self)
     VALUE self;
 {
     GtkWidgetClass* oclass = g_type_class_ref(CLASS2GTYPE(self));
-    gint n_properties;
+    guint n_properties;
     GParamSpec** props;
     VALUE inherited_too;
     VALUE ary;
@@ -783,9 +833,9 @@ widget_s_style_properties(argc, argv, self)
 
     ary = rb_ary_new();
     for (i = 0; i < n_properties; i++){
-        if (RTEST(inherited_too)
+        if (RVAL2CBOOL(inherited_too)
             || GTYPE2CLASS(props[i]->owner_type) == self)
-            rb_ary_push(ary, rb_str_new2(props[i]->name));
+            rb_ary_push(ary, CSTR2RVAL(props[i]->name));
     }
 
     g_free(props);
@@ -860,9 +910,26 @@ static VALUE
 widget_child_focus(self, direction)
     VALUE self, direction;
 {
-    return gtk_widget_child_focus(_SELF(self), RVAL2GENUM(direction, GTK_TYPE_DIRECTION_TYPE)) ? Qtrue : Qfalse;
+    return CBOOL2RVAL(gtk_widget_child_focus(_SELF(self), RVAL2GENUM(direction, GTK_TYPE_DIRECTION_TYPE)));
 }
 
+#if GTK_CHECK_VERSION(2,12,0)
+static VALUE
+widget_error_bell(self)
+    VALUE self;
+{
+    gtk_widget_error_bell(_SELF(self));
+    return self;
+}
+
+static VALUE
+widget_keynav_failed(self, direction)
+    VALUE self, direction;
+{
+    return CBOOL2RVAL(gtk_widget_keynav_failed(_SELF(self), RVAL2GENUM(direction, GTK_TYPE_DIRECTION_TYPE)));
+}
+#endif
+
 static VALUE
 widget_child_notify(self, child_property)
     VALUE self, child_property;
@@ -883,7 +950,7 @@ static VALUE
 widget_get_child_visible(self)
     VALUE self;
 {
-    return gtk_widget_get_child_visible(_SELF(self)) ? Qtrue : Qfalse;
+    return CBOOL2RVAL(gtk_widget_get_child_visible(_SELF(self)));
 }
 
 static VALUE
@@ -893,6 +960,43 @@ widget_get_settings(self)
     return GOBJ2RVAL(gtk_widget_get_settings(_SELF(self)));
 }
 
+#if GTK_CHECK_VERSION(2,2,0)
+static VALUE
+widget_get_clipboard(self, selection)
+    VALUE self, selection;
+{
+    return CLIPBOARD2RVAL(gtk_widget_get_clipboard(_SELF(self), RVAL2ATOM(selection)));
+}
+
+static VALUE
+widget_get_display(self)
+    VALUE self;
+{
+    return GOBJ2RVAL(gtk_widget_get_display(_SELF(self)));
+}
+
+static VALUE
+widget_get_root_window(self)
+    VALUE self;
+{
+    return GOBJ2RVAL(gtk_widget_get_root_window(_SELF(self)));
+}
+
+static VALUE
+widget_get_screen(self)
+    VALUE self;
+{
+    return GOBJ2RVAL(gtk_widget_get_screen(_SELF(self)));
+}
+
+static VALUE
+widget_has_screen(self)
+    VALUE self;
+{
+    return CBOOL2RVAL(gtk_widget_has_screen(_SELF(self)));
+}
+#endif
+
 /*
   Note this method is not gtk_widget_size_request()
 */
@@ -909,7 +1013,7 @@ static VALUE
 widget_set_child_visible(self, is_visible)
     VALUE self, is_visible;
 {
-    gtk_widget_set_child_visible(_SELF(self), RTEST(is_visible));
+    gtk_widget_set_child_visible(_SELF(self), RVAL2CBOOL(is_visible));
     return self;
 }
 
@@ -929,12 +1033,100 @@ widget_thaw_child_notify(self)
     return self;
 }
 
+#if GTK_CHECK_VERSION(2,4,0)
+static VALUE
+widget_set_no_show_all(self, no_show_all)
+    VALUE self, no_show_all;
+{
+    gtk_widget_set_no_show_all(_SELF(self), RVAL2CBOOL(no_show_all));
+    return self;
+}
+
+static VALUE
+widget_get_no_show_all(self)
+    VALUE self;
+{
+    return CBOOL2RVAL(gtk_widget_get_no_show_all(_SELF(self)));
+}
+
+static VALUE
+widget_list_mnemonic_labels(self)
+    VALUE self;
+{
+    return GLIST2ARYF(gtk_widget_list_mnemonic_labels(_SELF(self)));
+}
+
+static VALUE
+widget_add_mnemonic_label(self, label)
+    VALUE self, label;
+{
+    gtk_widget_add_mnemonic_label(_SELF(self), GTK_WIDGET(RVAL2GOBJ(label)));
+    return self;
+}
+
+static VALUE
+widget_remove_mnemonic_label(self, label)
+    VALUE self, label;
+{
+    gtk_widget_remove_mnemonic_label(_SELF(self), GTK_WIDGET(RVAL2GOBJ(label)));
+    return self;
+}
+#endif
+
+#if GTK_CHECK_VERSION(2,12,0)
+static VALUE
+widget_set_tooltip_window(self, custom_window)
+    VALUE self, custom_window;
+{
+    gtk_widget_set_tooltip_window(_SELF(self), GTK_WINDOW(RVAL2GOBJ(custom_window)));
+    return self;
+}
+
+static VALUE
+widget_get_tooltip_window(self)
+    VALUE self;
+{
+    return GOBJ2RVAL(gtk_widget_get_tooltip_window(_SELF(self)));
+}
+
+static VALUE
+widget_trigger_tooltip_query(self)
+    VALUE self;
+{
+    gtk_widget_trigger_tooltip_query(_SELF(self));
+    return self;
+}
+
+#endif
+
+#if GTK_CHECK_VERSION(2,10,0)
+static VALUE
+widget_get_action(self)
+    VALUE self;
+{
+    return GOBJ2RVAL(gtk_widget_get_action(_SELF(self)));
+}
+
+static VALUE
+widget_is_composited(self)
+    VALUE self;
+{
+    return CBOOL2RVAL(gtk_widget_is_composited(_SELF(self)));
+}
+#endif
+
 static VALUE
 widget_window(self)
     VALUE self;
 {
-    GdkWindow* window = _SELF(self)->window;
-    return window ? GOBJ2RVAL(window) : Qnil;
+    return GOBJ2RVAL(_SELF(self)->window);
+}
+
+static VALUE
+widget_set_window(VALUE self, VALUE window)
+{
+    _SELF(self)->window = RVAL2GOBJ(window);
+    return self;
 }
 
 #define DEFINE_IS_WIDGET(STATE) \
@@ -983,18 +1175,28 @@ widget_get_requisition(self)
     return rb_ary_new3(2, INT2NUM(req.width), INT2NUM(req.height));
 }
 
+static VALUE
+widget_set_requisition(self, w, h)
+    VALUE self, w, h;
+{
+    GtkRequisition *r = &(_SELF(self)->requisition);
+    r->width  = NUM2INT(w);
+    r->height = NUM2INT(h);
+    return self;    
+}
+
 static VALUE
 widget_state(self)
     VALUE self;
 {
-    return INT2FIX(_SELF(self)->state);
+    return GENUM2RVAL(_SELF(self)->state, GTK_TYPE_STATE_TYPE);
 }
 
 static VALUE
 widget_saved_state(self)
     VALUE self;
 {
-    return INT2FIX(_SELF(self)->saved_state);
+    return GENUM2RVAL(_SELF(self)->saved_state, GTK_TYPE_STATE_TYPE);
 }
 
 static VALUE
@@ -1014,6 +1216,7 @@ widget_signal_size_allocate(num, values)
     GtkAllocation* alloc = (GtkAllocation*)g_value_get_boxed(&values[1]);
     return rb_ary_new3(2, GVAL2RVAL(&values[0]), BOXED2RVAL(alloc, GTK_TYPE_ALLOCATION));
 }
+
 void 
 Init_gtk_widget()
 {
@@ -1040,6 +1243,9 @@ Init_gtk_widget()
     rb_define_method(gWidget, "unrealize", widget_unrealize, 0);
     rb_define_method(gWidget, "queue_draw", widget_queue_draw, 0);
     rb_define_method(gWidget, "queue_resize", widget_queue_resize, 0);
+#if GTK_CHECK_VERSION(2,4,0)
+    rb_define_method(gWidget, "queue_resize_no_redraw", widget_queue_resize_no_redraw, 0);
+#endif
     rb_define_method(gWidget, "size_request", widget_size_request, 0);
     rb_define_method(gWidget, "child_requisition", widget_get_child_requisition, 0);
     rb_define_method(gWidget, "size_allocate", widget_size_allocate, 1);
@@ -1047,6 +1253,9 @@ Init_gtk_widget()
     rb_define_method(gWidget, "remove_accelerator", widget_remove_accelerator, 3);
     rb_define_method(gWidget, "set_accel_path", widget_set_accel_path, 2);
     rb_define_method(gWidget, "accel_closures", widget_list_accel_closures, 0);
+#if GTK_CHECK_VERSION(2,4,0)
+    rb_define_method(gWidget, "can_activate_accel?", widget_can_activate_accel, 1);
+#endif
     rb_define_method(gWidget, "event", widget_event, 1);
     rb_define_method(gWidget, "activate", widget_activate, 0);
     rb_define_method(gWidget, "reparent", widget_reparent, 1);
@@ -1077,6 +1286,9 @@ Init_gtk_widget()
     rb_define_method(gWidget, "set_direction", widget_set_direction, 1);
     rb_define_method(gWidget, "direction", widget_get_direction, 0);
     rb_define_method(gWidget, "shape_combine_mask", widget_shape_combine_mask, 3);
+#if GTK_CHECK_VERSION(2,10,0)
+    rb_define_method(gWidget, "input_shape_combine_mask", widget_input_shape_combine_mask, 3);
+#endif
     rb_define_method(gWidget, "path", widget_path, 0);
     rb_define_method(gWidget, "class_path", widget_class_path, 0);
     rb_define_method(gWidget, "composite_name", widget_get_composite_name, 0);
@@ -1087,10 +1299,13 @@ Init_gtk_widget()
     rb_define_method(gWidget, "modify_text", widget_modify_text, 2);
     rb_define_method(gWidget, "modify_base", widget_modify_base, 2);
     rb_define_method(gWidget, "modify_font", widget_modify_font, 1);
+#if GTK_CHECK_VERSION(2,12,0)
+    rb_define_method(gWidget, "modify_cursor", widget_modify_cursor, 2);
+#endif
     rb_define_method(gWidget, "create_pango_context", widget_create_pango_context, 0);
     rb_define_method(gWidget, "pango_context", widget_get_pango_context, 0);
     rb_define_method(gWidget, "create_pango_layout", widget_create_pango_layout, -1);
-    rb_define_method(gWidget, "render_icon", widget_render_icon, 3);
+    rb_define_method(gWidget, "render_icon", widget_render_icon, -1);
     rb_define_method(gWidget, "queue_draw_area", widget_queue_draw_area, 4);
     rb_define_method(gWidget, "reset_shapes", widget_reset_shapes, 0);
     rb_define_method(gWidget, "set_double_buffered", widget_set_double_buffered, 1);
@@ -1103,18 +1318,47 @@ Init_gtk_widget()
     rb_define_method(gWidget, "style_get_property", widget_style_get_property, 1);
     rb_define_method(gWidget, "accessible", widget_get_accessible, 0);
     rb_define_method(gWidget, "child_focus", widget_child_focus, 1);
+#if GTK_CHECK_VERSION(2,12,0)
+    rb_define_method(gWidget, "error_bell", widget_error_bell, 0);
+    rb_define_method(gWidget, "keynav_failed", widget_keynav_failed, 0);
+#endif
     rb_define_method(gWidget, "child_notify", widget_child_notify, 1);
     rb_define_method(gWidget, "freeze_child_notify", widget_freeze_child_notify, 0);
     rb_define_method(gWidget, "child_visible?", widget_get_child_visible, 0);
     rb_define_method(gWidget, "settings", widget_get_settings, 0);
+#if GTK_CHECK_VERSION(2,2,0)
+    rb_define_method(gWidget, "get_clipboard", widget_get_clipboard, 1);
+    rb_define_method(gWidget, "display", widget_get_display, 0);
+    rb_define_method(gWidget, "root_window", widget_get_root_window, 0);
+    rb_define_method(gWidget, "screen", widget_get_screen, 0);
+    rb_define_method(gWidget, "has_screen?", widget_has_screen, 0);
+#endif
     rb_define_method(gWidget, "set_child_visible", widget_set_child_visible, 1);
     rb_define_method(gWidget, "get_size_request", widget_get_size_request, 0);
     rb_define_method(gWidget, "set_size_request", widget_set_size_request, 2);
     rb_define_method(gWidget, "thaw_child_notify", widget_thaw_child_notify, 0);
+#if GTK_CHECK_VERSION(2,4,0)
+    rb_define_method(gWidget, "set_no_show_all", widget_set_no_show_all, 1);
+    rb_define_method(gWidget, "no_show_all?", widget_get_no_show_all, 0);
+    rb_define_method(gWidget, "mnemonic_labels", widget_list_mnemonic_labels, 0);
+    rb_define_method(gWidget, "add_mnemonic_label", widget_add_mnemonic_label, 1);
+    rb_define_method(gWidget, "remove_mnemonic_label", widget_remove_mnemonic_label, 1);
+#endif
+#if GTK_CHECK_VERSION(2,12,0)
+    rb_define_method(gWidget, "set_tooltip_window", widget_set_tooltip_window, 1);
+    rb_define_method(gWidget, "tooltip_window", widget_get_tooltip_window, 0);
+    rb_define_method(gWidget, "trigger_tooltip_query", widget_trigger_tooltip_query, 0);
+#endif
+#if GTK_CHECK_VERSION(2,10,0)
+    rb_define_method(gWidget, "action", widget_get_action, 0);
+    rb_define_method(gWidget, "composited?", widget_is_composited, 0);
+#endif
     rb_define_method(gWidget, "window", widget_window, 0);
+    rb_define_method(gWidget, "set_window", widget_set_window, 1);
     rb_define_method(gWidget, "allocation", widget_get_allocation, 0);
     rb_define_method(gWidget, "set_allocation", widget_set_allocation, 4);
     rb_define_method(gWidget, "requisition", widget_get_requisition, 0);
+    rb_define_method(gWidget, "set_requisition", widget_set_requisition, 2);
     rb_define_method(gWidget, "state", widget_state, 0);
     rb_define_method(gWidget, "saved_state", widget_saved_state, 0);
 
@@ -1162,7 +1406,7 @@ Init_gtk_widget()
     G_DEF_CONSTANTS(gWidget, GTK_TYPE_TEXT_DIRECTION, "GTK_");
 
     /* Special signals */
-    G_DEF_SIGNAL_FUNC(gWidget, "size-request", widget_signal_size_request);
-    G_DEF_SIGNAL_FUNC(gWidget, "size-allocate", widget_signal_size_allocate);
+    G_DEF_SIGNAL_FUNC(gWidget, "size-request", (GValToRValSignalFunc)widget_signal_size_request);
+    G_DEF_SIGNAL_FUNC(gWidget, "size-allocate", (GValToRValSignalFunc)widget_signal_size_allocate);
     
 }
