@@ -16,16 +16,18 @@ require 'mkmf-gnome2'
 PKGConfig.have_package('pango') or exit 1
 setup_win32(PACKAGE_NAME)
 
-have_func("pango_layout_iter_get_type")
-have_func("pango_layout_set_ellipsize")
-have_func("pango_layout_get_font_description")
-have_func("pango_render_part_get_type")
-have_func("pango_attr_strikethrough_color_new")
-have_func("pango_attr_underline_color_new")
-have_func("pango_glyph_item_free")
-have_func("pango_attr_iterator_get_attrs")
-have_func("pango_itemize_with_base_dir")
-have_func("pango_font_family_is_monospace")
+pango_header = "pango/pango.h"
+have_func("pango_layout_iter_get_type", pango_header)
+have_func("pango_layout_set_ellipsize", pango_header)
+have_func("pango_layout_get_font_description", pango_header)
+have_func("pango_render_part_get_type", pango_header)
+have_func("pango_attr_strikethrough_color_new", pango_header)
+have_func("pango_attr_underline_color_new", pango_header)
+have_func("pango_glyph_item_free", pango_header)
+have_func("pango_glyph_item_get_type", pango_header)
+have_func("pango_attr_iterator_get_attrs", pango_header)
+have_func("pango_itemize_with_base_dir", pango_header)
+have_func("pango_font_family_is_monospace", pango_header)
 
 PKGConfig.have_package('pangocairo')
 if have_header('rb_cairo.h')
@@ -34,7 +36,7 @@ if have_header('rb_cairo.h')
       puts "Error! Set CAIRO_PATH."
       exit 1
     end
-    add_depend_package("cairo", "packages/cairo/ext", ENV["CAIRO_PATH"])
+    add_depend_package("cairo", "src", ENV["CAIRO_PATH"])
     $defs << "-DRUBY_CAIRO_PLATFORM_WIN32"
   end
 end
@@ -46,7 +48,8 @@ add_distcleanfile("rbpangoinits.c")
 make_version_header("PANGO", "pango")
 
 create_makefile_at_srcdir(PACKAGE_NAME, SRCDIR, "-DRUBY_PANGO_COMPILATION") {
-  system("#{$ruby} #{SRCDIR}/makeinits.rb #{SRCDIR}/*.c > rbpangoinits.c")
+  SRCDIR_QUOTED = SRCDIR.gsub(' ', '\ ')
+  system("#{$ruby} #{SRCDIR_QUOTED}/makeinits.rb #{SRCDIR_QUOTED}/*.c > rbpangoinits.c")
 }
 
 create_top_makefile
